/*---------------------------------------------------------------------------+
|	This file is part of MuMail, Copyright (c) 1992-1993 by
|	Muhammad M. Saggaf. All rights reserved.
|
|	See the file COPYING (1-COPYING) or the manual page mumail(1)
|	for a full statement of rights and permissions.
+---------------------------------------------------------------------------*/

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/AsciiText.h>
#include <unistd.h>
#include <string.h>
#include "MuWin.h"
#include "MuGeneric.h"
#include "MmDecl.h"

#define DIALOG_END_CHAR ('\200')

typedef struct {
  SCREEN          *scn;
  XtCallbackProc  callBack;
  XtPointer       clientData;
  String          dialogString;
} _dialogRec;

/*---------------------------------------------------------------------------+
| Message routines.
+---------------------------------------------------------------------------*/

void
ClearMessageTimerCallback(clientData, timerId)
	 XtPointer  clientData;
	 XtIntervalId *timerId;
{
  SCREEN *scn = (SCREEN*)clientData;
  XtVaSetValues(scn->labelW, XtNlabel, "", NULL);
}

void
Message(scn, msg, persistenceTime)
	 SCREEN   *scn;
	 String    msg;
	 int       persistenceTime;
{
  static XtIntervalId timerId[MAX_SCREENS];

  XtVaSetValues(scn->labelW, XtNlabel, msg, NULL);

  if (timerId[scn->number]) XtRemoveTimeOut(timerId[scn->number]);
  if (persistenceTime) timerId[scn->number] = 
	XtAppAddTimeOut(XtWidgetToApplicationContext(scn->parentW),
					persistenceTime*1000, ClearMessageTimerCallback, 
					(XtPointer)scn);
}

/*---------------------------------------------------------------------------+
| Dialog input routines.
+---------------------------------------------------------------------------*/

void
DisableDialogInput(scn)
	 SCREEN         *scn;
{
  XtRemoveAllCallbacks(XawTextGetSource(scn->dialog.valueW), XtNcallback);
  XtRemoveAllCallbacks(scn->dialog.yesW, XtNcallback);
  XtRemoveAllCallbacks(scn->dialog.cancelW, XtNcallback);

  XtVaSetValues(scn->dialog.valueW, XtNdisplayCaret, False, XtNeditType, 
				XawtextRead, XtNstring, "", NULL);
  XtVaSetValues(scn->dialog.yesW, XtNmappedWhenManaged, False, NULL);
  XtVaSetValues(scn->dialog.cancelW, XtNmappedWhenManaged, False, NULL);

  scn->dialog.active = False;
}

void
CancelDialogInputCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _dialogRec *dialogRec = (_dialogRec*)clientData;
  SCREEN     *scn = dialogRec->scn;

  DisableDialogInput(scn);
  Message(scn, "Cancelled", MSG_CLEAR_DEFAULT);
}

int
CallDialogCallback(dialogRec, dialogString)
	 _dialogRec           *dialogRec;
	 String               dialogString;
{
  SCREEN            *scn = dialogRec->scn;

  /* Empty input -> cancel */

  if (*dialogString == CNULL) {
	DisableDialogInput(scn);
	Message(scn, "Empty string, cancelled", MSG_CLEAR_DEFAULT);
	return -1;
  }

  if (dialogRec->dialogString) XtFree(dialogRec->dialogString);
  dialogRec->dialogString = XtNewString(dialogString);

  DisableDialogInput(scn);
  (*dialogRec->callBack)(scn->dialog.valueW, dialogRec->clientData, 
						 (XtPointer)dialogRec->dialogString);
  return 0;
}

void
MonitorDialogInputCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _dialogRec           *dialogRec = (_dialogRec*)clientData;
  String               dialogString,
                       tmpPtr;

  XtVaGetValues(widget, XtNstring, &dialogString, NULL);
  if ((tmpPtr = strchr(dialogString, DIALOG_END_CHAR)) == NULL) return;
  while(*tmpPtr) {*tmpPtr = *(tmpPtr+1); tmpPtr++;}
  CallDialogCallback(dialogRec, dialogString);
}

void
CallDialogCallbackCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _dialogRec           *dialogRec = (_dialogRec*)clientData;
  SCREEN               *scn = dialogRec->scn;
  String               dialogString;

  XtVaGetValues(scn->dialog.valueW, XtNstring, &dialogString, NULL);
  CallDialogCallback(dialogRec, dialogString);
}

int
DialogInputAndDispatch(scn, callBack, clientData)
	 SCREEN         *scn;
	 XtCallbackProc  callBack;
	 XtPointer       clientData;
{
  static _dialogRec          dialogRec[MAX_SCREENS];

  if (scn->dialog.active) return -1;

  dialogRec[scn->number].scn = scn;
  dialogRec[scn->number].callBack = callBack;
  dialogRec[scn->number].clientData = clientData;

  scn->dialog.active = True;

  XtVaSetValues(scn->dialog.valueW, XtNdisplayCaret, True, XtNeditType, 
				XawtextEdit, XtNstring, dialogRec[scn->number].dialogString ? 
				dialogRec[scn->number].dialogString : "", NULL);
  XtAddCallback(XawTextGetSource(scn->dialog.valueW), XtNcallback, 
				MonitorDialogInputCallback, 
				(XtPointer)&dialogRec[scn->number]);

  XtAddCallback(scn->dialog.yesW, XtNcallback, CallDialogCallbackCallback, 
				(XtPointer)&dialogRec[scn->number]);
  XtAddCallback(scn->dialog.cancelW, XtNcallback, CancelDialogInputCallback, 
				(XtPointer)&dialogRec[scn->number]);

  XtVaSetValues(scn->dialog.yesW, XtNlabel, "OK", 
				XtNmappedWhenManaged, True, NULL);
  XtVaSetValues(scn->dialog.cancelW, XtNlabel, "Cancel", 
				XtNmappedWhenManaged, True, NULL);
  return 0;
}

/*---------------------------------------------------------------------------+
| Boolean input routines.
+---------------------------------------------------------------------------*/

void
DisableBooleanInput(scn)
	 SCREEN         *scn;
{
  XtRemoveAllCallbacks(scn->dialog.yesW, XtNcallback);
  XtRemoveAllCallbacks(scn->dialog.cancelW, XtNcallback);

  XtVaSetValues(scn->dialog.yesW, XtNmappedWhenManaged, False, NULL);
  XtVaSetValues(scn->dialog.cancelW, XtNmappedWhenManaged, False, NULL);

  scn->dialog.active = False;
}

void
CancelBooleanInputCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _dialogRec           *dialogRec = (_dialogRec*)clientData;
  SCREEN               *scn = dialogRec->scn;

  DisableBooleanInput(scn);
  Message(scn, "Unconfirmed", MSG_CLEAR_DEFAULT);
}

void
CallBooleanCallbackCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  _dialogRec           *dialogRec = (_dialogRec*)clientData;
  SCREEN               *scn = dialogRec->scn;

  DisableBooleanInput(scn);
  (*dialogRec->callBack)(scn->dialog.valueW, dialogRec->clientData, callData);
}

int
BooleanInputAndDispatch(scn, callBack, clientData)
	 SCREEN         *scn;
	 XtCallbackProc  callBack;
	 XtPointer       clientData;
{
  static _dialogRec          dialogRec[MAX_SCREENS];

  if (scn->dialog.active) return -1;

  dialogRec[scn->number].scn = scn;
  dialogRec[scn->number].callBack = callBack;
  dialogRec[scn->number].clientData = clientData;

  scn->dialog.active = True;

  XtAddCallback(scn->dialog.yesW, XtNcallback, CallBooleanCallbackCallback, 
				(XtPointer)&dialogRec[scn->number]);
  XtAddCallback(scn->dialog.cancelW, XtNcallback, CancelBooleanInputCallback, 
				(XtPointer)&dialogRec[scn->number]);

  XtVaSetValues(scn->dialog.yesW, XtNlabel, "Yes", XtNmappedWhenManaged, 
				True, NULL);
  XtVaSetValues(scn->dialog.cancelW, XtNlabel, "No", XtNmappedWhenManaged, 
				True, NULL);
  return 0;
}

String
FileInMuMailDir(fileName)
	 String fileName;
{
  return FileInHomeDir(FmtString1(".mumail/%s", fileName));
}


#if 0
/*---------------------------------------------------------------------------+
| Delayed procedure execution routines.
+---------------------------------------------------------------------------*/

void
ExecDelayedCallback(clientData, source, id)
     XtPointer      clientData;
	 int            *source;
	 XtInputId      *id;
{
  _clientDataRec  *clientDataRec = (_clientDataRec*)clientData;
  char            dummy;

  printf("got it\n");
  ReadFromPipe(source, &dummy, sizeof(char));
  (*(XtCallbackProc)clientDataRec->field[0])((Widget)clientDataRec->field[1],
											 clientDataRec->field[2],
											 clientDataRec->field[3]);
}

void
DelayedCallback(scn, widget, callback, clientData, callData)
	 SCREEN         *scn;
	 XtCallbackProc callback;
     Widget         widget;
     XtPointer      clientData,
                    callData;
{
  static int            pd[2];
  static _clientDataRec clientDataRec;
  char                  dummy = CNULL;

  clientDataRec.field[0] = (XtPointer)callback;
  clientDataRec.field[1] = (XtPointer)widget;
  clientDataRec.field[2] = clientData;
  clientDataRec.field[3] = callData;

  if (pd[0] == 0) {
	pipe(pd);
	XtAppAddInput(XtWidgetToApplicationContext(scn->parentW), pd[0], 
				  (XtPointer)XtInputReadMask, ExecDelayedCallback, 
				  (XtPointer)&clientDataRec);
  }

  WriteToPipe(pd, &dummy, sizeof(char));
}
#endif
